package
{
	import flash.display.Bitmap;
	import flash.geom.Point;
	import flash.ui.Mouse;
	
	import org.flixel.*;
	import org.flixel.FlxSprite;
	import org.flixel.FlxState;
	import org.flixel.FlxU;
	
	public class GameState extends FlxState
	{
		[Embed(source = "data/SectionTitle.mp3")] public static var SndTitle:Class;
		[Embed(source = "data/Section1.mp3")] public static var SndSection1:Class;
		[Embed(source = "data/Section2.mp3")] public static var SndSection2:Class;

		[Embed(source = "data/room.png")] public static var ImgRoom:Class;
		[Embed(source = "data/full.png")] public static var ImgFull:Class;
		[Embed(source = "data/empty.png")] public static var ImgEmpty:Class;
		[Embed(source = "data/selector.png")] public static var ImgSelector:Class;
		[Embed(source = "data/flowergot.png")] public static var ImgFlowerGot:Class;
		[Embed(source = "data/selectorFull.png")] public static var ImgSelectorOn:Class;

		public var fadedin:Boolean;
		public const tileSize:int=32;
		
		public var flowerArray:Array;
		public var score:int;
		public var titleText:FlxText;
		public var fading:Boolean;
		
		public function incrementScore():void
		{
			score++;
			for (var i:int=0;i<9;i++)
			{
				flowerArray[i].visible = 8-i<score;
			}
			
			if (score==9)
			{
				levelCompleted();
			}
		}
		
		public function levelCompleted():void
		{
			FlxG.fade.start(0xff000000,1,onCompleted);
			fadedin=false;
		}
		
		public function levelFailed():void
		{
			FlxG.fade.start(0xff000000,1,onFailed);
			fadedin=false;
		}
		
		
		public function onFailed():void
		{		
			FlxG.state = new RPGState();
		}
		
		
		public function onCompleted():void
		{
			switch (strategy)
			{
				case 0:
					RPGState.finished1=true;
					break;
				case 1:
					RPGState.finished2=true;
					break;
				case 2:
					RPGState.finished3=true;
					break;
				case 3:
					RPGState.finished4=true;
					break;
			}

			FlxG.state = new RPGState();
		}
		
		override public function create():void
		{		
			fadedin=false;
			this._time=0;
			
			var bg:FlxSprite = new FlxSprite(0,0,ImgRoom);
			add(bg);
			
			
			flowerArray = new Array();
			for (var i:int=0;i<9;i++)
			{
				var fx:Number=246;
				var fy:Number=94+i*41;
				var s:FlxSprite = new FlxSprite(fx,fy,ImgFlowerGot);
				flowerArray.push(s);
				s.visible=false;
				this.add(s);
			}
			
			score=-1;
			incrementScore();
			
			selectors = new FlxGroup();
			tiles = new FlxGroup();
			
			areas=generateLevel();			
			updateSelectors();
			generateLevelDisplay();
			
			titleText = new FlxText(69,46,382,"");
			switch(strategy)
			{
				case 0:
					titleText.text="Chamber of the Northern Winds";
					break;
				case 1:
					titleText.text="Chamber of the Southern Winds";
					break;
				case 2:
					titleText.text="Chamber of the Western Winds";
					break;
				case 3:
					titleText.text="Chamber of the Eastern Winds";
					break;
			}
			titleText.alignment="center";
			titleText.color=0xff2b281d;
			titleText.size=16;
			add(titleText);
			add(tiles);	
			add(selectors);	
			FlxG.flash.start(0xff000000,1,FlashFinished);
		}
		public function FlashFinished():void
		{
			fadedin=true;
		}
		
		public var selectors:FlxGroup;
		public var tiles:FlxGroup;
		
		public var sx:int;
		public var sy:int;
		
		public var bOnRightPoint:Boolean;
		
		public function updateSelectors():void
		{
			if (sx<0)
				sx=0;
			else if (sx>=areas[0].length)
				sx=areas[0].length-1;
			
			if (sy<0)
				sy=0;
			else if (sy>=areas[0].length)
				sy=areas[0].length-1;
			
			for (var i:int=selectors.members.length-1;i>=0;i--)
			{
				selectors.members[i].kill();
				selectors.remove(selectors.members[i],true);
			}

			bOnRightPoint=true;

			for (i=0;i<areas.length;i++)
			{
				if (areas[i][sx][sy]==0)
				{
					bOnRightPoint=false;
					break;
				}
			}
			
			for (i=0;i<areas.length;i++)
			{
				var displayCount:int = areas.length;
				var areaSize:int = areas[0].length;
				var x:FlxSprite = new FlxSprite(offsetX + (areaSize*32+32)*(i%2)+sx*32,offsetY +(areaSize*32+12)*Math.floor(i/2)+sy*32);
											
				//if (bOnRightPoint)				
				//{
			//		x.loadGraphic(ImgSelectorOn,true,false,32,32);
			//		x.addAnimation("flicker",[0,1],6,true);
			//		x.play("flicker");
			//	}
			//	else
				{
					x.loadGraphic(ImgSelector);
				}
					
				selectors.add(x);			
			}
		}
		
		public static const myWidth:int=6;
		public static const areaCount:int=4;
		public static var strategy:int;
			
		
		public static function generateLevel():Array
		{		
			var newArea:Array = new Array();
			
			var tx:int=Math.floor(Math.random()*myWidth);
			var ty:int=Math.floor(Math.random()*myWidth);
			
			if (strategy==1)
			{
				tx = 1+Math.floor(Math.random()*(myWidth-2));
				ty = 1+Math.floor(Math.random()*(myWidth-2));
			}
			
			for (var i:int=0;i<areaCount;i++)
			{
				var sector:Array = new Array();
				
				for (var j:int=0;j<myWidth;j++)
				{
					var row:Array = new Array();
					for (var k:int=0;k<myWidth;k++)
					{
						if (j==tx && k==ty)
						{
							row.push(1);
						}
						else
						{
							row.push(0);						
						}
					}
					sector.push(row);
				}
				newArea.push(sector);
			}			
			
			
			for (i=0;i<areaCount*myWidth*myWidth;i++)
			{
				var a:int,b:int,c:int;
				
				switch (strategy)
				{
					case 0:
						a = Math.floor(Math.random()*areaCount);
						b = Math.floor(Math.random()*(myWidth-1));
						c = Math.floor(Math.random()*(myWidth-1));
						if (b>=tx)
						{
							b++;
						}
						
						if (c>=ty)
						{
							c++;
						}
						break;
					case 1://pure strategy...for smaller boards?
						/// NOPE SYMMETRY NOW
						a = Math.floor(Math.random()*areaCount);
						b = Math.floor(Math.random()*(myWidth));
						c = Math.floor(Math.random()*(myWidth));

						if (b==tx&&c==ty || Math.random()>0.3)
							continue;
						break;
					case 2://circle missing
						a = Math.floor(Math.random()*areaCount);
						b = Math.floor(Math.random()*(myWidth));
						c = Math.floor(Math.random()*(myWidth));
						
						if ((b-tx)*(b-tx)+(c-ty)*(c-ty)<7)
							continue;
						break;
					case 3://sparse
						a = Math.floor(Math.random()*areaCount);
						b = Math.floor(Math.random()*(myWidth));
						c = Math.floor(Math.random()*(myWidth));
						
						if (b==tx&&c==ty || Math.random()>0.1)
							continue;
						break;
					default:
						break;
				}
				
				
				if (newArea[a][b][c]==1)
					continue;				
				
				var bEmptySpace:Boolean=false;
				for (j=0;j<areaCount;j++)
				{
					if (j==a)
						continue;
					
					if (newArea[j][b][c]==0)
						bEmptySpace=true;
				}
				
				if (bEmptySpace)
				{
					
					if (strategy==1)
					{
						//place symmetrically about point
						var points:Array = new Array();
						points.push(new Point(b,c));
						var dx:int = points[points.length-1].x-tx;
						var dy:int = points[points.length-1].y-ty;
						
						
						points.push(new Point(tx-dx,ty-dy));
						points.push(new Point(tx+dx,ty-dy));
						points.push(new Point(tx-dx,ty+dy));
						
						
						for (j=points.length-1;j>=0;j--)
						{
							var newx:int=points[j].x;
							var newy:int=points[j].y;
							if (newx<0 || newx>=6 || newy<0 || newy>=6)
							{
								points.splice(j,1);								
							}
							else
							{
								//trace(newx,newy);
							}
						}
						
						for (j=points.length-1;j>=0;j--)
						{
							newArea[a][points[j].x][points[j].y]=1;
						}
						
					}
					else
					{
						newArea[a][b][c]=1;
					}
				}
			}
			
			return newArea;
		}
		
		public const offsetX:Number = 52;
		public const offsetY:Number = 75;
		
		
		public function generateLevelDisplay():void
		{			
			var displayCount:int = areas.length;
			var areaSize:int = areas[0].length;
			
			for (i=tiles.members.length-1;i>=0;i--)
			{
				tiles.remove(tiles.members[i],true);
			}
			
			for (var i:int=0;i<displayCount;i++)
			{
				
				for (var j:int=0;j<areaSize;j++)
				{
					for (var k:int=0;k<areaSize;k++)
					{
						var tileType:int = areas[i][j][k];		
						var x:FlxSprite = new FlxSprite(offsetX+(areaSize*32+32)*(i%2)+j*32,offsetY+(areaSize*32+12)*Math.floor(i/2)+k*32);
						switch(tileType)
						{
							case 0:
								x.loadGraphic(ImgEmpty);
								break;
							case 1:
								x.loadGraphic(ImgFull);
								break;
						}
						tiles.add(x);
					}
				}
				
			}
		}
		
		public var _time:Number;
		
		public var areas:Array;
		
		override public function update():void
		{
			if (!fadedin)
			{
				super.update();
				return;
			}		
			
			if (FlxG.keys.justPressed("X") || FlxG.keys.justPressed("SPACE") || FlxG.keys.justPressed("F"))
			{
				if (bOnRightPoint)
				{
					incrementScore();
					
					if (score<=8)
					{
						areas=generateLevel();						
						updateSelectors();
						generateLevelDisplay();
					}
					FlxG.flash.start(0xffd6bf8f,0.5);
				}
				else
				{
					score-=2;
					if (score<0)
						score=-1;
					incrementScore();
				}
				super.update();
				return;
			}
			
			if (FlxG.keys.justPressed("UP")||FlxG.keys.justPressed("W"))
			{
				sy--;
				updateSelectors();
			}
			
			if (FlxG.keys.justPressed("DOWN")||FlxG.keys.justPressed("S"))
			{
				sy++;
				updateSelectors();				
			}
			
			if (FlxG.keys.justPressed("LEFT")||FlxG.keys.justPressed("A"))
			{
				sx--;
				updateSelectors();				
			}
			
			if (FlxG.keys.justPressed("RIGHT")||FlxG.keys.justPressed("D"))
			{
				sx++;
				updateSelectors();				
			}
			
			if (FlxG.keys.justPressed("ESCAPE"))
			{
				levelFailed();	
			}
			
			_time+=FlxG.elapsed;
			super.update();
		}
	}
}

class Point 
{
	public var x:int;
	public var y:int;
	function Point(x_:int,y_:int)
	{
		x=x_;
		y=y_;
	}
}